# encoding: utf-8

"""Release information about the Turbo-Mail mail queue."""

version = "1.1"
description = "Multi-threaded mail queue manager for TurboGears applications."
author = "Top Floor Computer Systems Ltd."
email = "webmaster@topfloor.ca"
url = "http://www.topfloor.ca/turbomail/"
download_url = "http://www.topfloor.ca/turbomail/"
copyright = "© 2006 Top Floor Computer Systems Ltd."
license="MIT"
